#!/usr/bin/perl -w

use strict;

use Test::More tests => 10;

BEGIN 
{
use_ok("Games::LMSolve::Numbers");
use_ok("Games::LMSolve::Base");
use_ok("Games::LMSolve::Plank::Base");
use_ok("Games::LMSolve::Plank::Hex");
use_ok("Games::LMSolve::Alice");
use_ok("Games::LMSolve::Tilt::Base");
use_ok("Games::LMSolve::Tilt::Single");
use_ok("Games::LMSolve::Tilt::Multi");
use_ok("Games::LMSolve::Tilt::RedBlue");
use_ok("Games::LMSolve::Input");
}
